@@ -1,15 +1,16 @@
-#!/usr/bin/python
-
 from __future__ import unicode_literals
 
+import re
+
+from .metrics_core import Metric
+from .samples import Sample
+
 try:
     import StringIO
 except ImportError:
     # Python 3
     import io as StringIO
 
-from . import core
-
 
 def text_string_to_metric_families(text):
     """Parse Prometheus text format from a unicode string.
@@ -19,11 +20,37 @@ def text_string_to_metric_families(text):
     for metric_family in text_fd_to_metric_families(StringIO.StringIO(text)):
         yield metric_family
 
+
+ESCAPE_SEQUENCES = {
+    '\\\\': '\\',
+    '\\n': '\n',
+    '\\"': '"',
+}
+
+
+def replace_escape_sequence(match):
+    return ESCAPE_SEQUENCES[match.group(0)]
+
+
+HELP_ESCAPING_RE = re.compile(r'\\[\\n]')
+ESCAPING_RE = re.compile(r'\\[\\n"]')
+
+
 def _replace_help_escaping(s):
-    return s.replace("\\n", "\n").replace('\\\\', '\\')
+    return HELP_ESCAPING_RE.sub(replace_escape_sequence, s)
+
 
 def _replace_escaping(s):
-    return s.replace("\\n", "\n").replace('\\\\', '\\').replace('\\"', '"')
+    return ESCAPING_RE.sub(replace_escape_sequence, s)
+
+
+def _is_character_escaped(s, charpos):
+    num_bslashes = 0
+    while (charpos > num_bslashes
+           and s[charpos - 1 - num_bslashes] == '\\'):
+        num_bslashes += 1
+    return num_bslashes % 2 == 1
+
 
 def _parse_labels(labels_string):
     labels = {}
@@ -52,17 +79,17 @@ def _parse_labels(labels_string):
             i = 0
             while i < len(value_substr):
                 i = value_substr.index('"', i)
-                if value_substr[i - 1] != "\\":
+                if not _is_character_escaped(value_substr, i):
                     break
                 i += 1
 
-            # The label value is inbetween the first and last quote
+            # The label value is between the first and last quote
             quote_end = i + 1
             label_value = sub_labels[quote_start:quote_end]
             # Replace escaping if needed
             if escaping:
                 label_value = _replace_escaping(label_value)
-            labels[label_name.strip()] = label_value.strip()
+            labels[label_name.strip()] = label_value
 
             # Remove the processed label from the sub-slice for next iteration
             sub_labels = sub_labels[quote_end + 1:]
@@ -76,15 +103,17 @@ def _parse_labels(labels_string):
 
 
 # If we have multiple values only consider the first
-def _parse_value(s):
+def _parse_value_and_timestamp(s):
     s = s.lstrip()
     separator = " "
     if separator not in s:
         separator = "\t"
-    i = s.find(separator)
-    if i == -1:
-        return s
-    return s[:i]
+    values = [value.strip() for value in s.split(separator) if value.strip()]
+    if not values:
+        return float(s), None
+    value = float(values[0])
+    timestamp = (float(values[-1]) / 1000) if len(values) > 1 else None
+    return value, timestamp
 
 
 def _parse_sample(text):
@@ -96,8 +125,8 @@ def _parse_sample(text):
         # We ignore the starting curly brace
         label = text[label_start + 1:label_end]
         # The value is after the label end (ignoring curly brace and space)
-        value = float(_parse_value(text[label_end + 2:]))
-        return name, _parse_labels(label), value
+        value, timestamp = _parse_value_and_timestamp(text[label_end + 2:])
+        return Sample(name, _parse_labels(label), value, timestamp)
 
     # We don't have labels
     except ValueError:
@@ -108,8 +137,8 @@ def _parse_sample(text):
         name_end = text.index(separator)
         name = text[:name_end]
         # The value is after the name
-        value = float(_parse_value(text[name_end:]))
-        return name, {}, value
+        value, timestamp = _parse_value_and_timestamp(text[name_end:])
+        return Sample(name, {}, value, timestamp)
 
 
 def text_fd_to_metric_families(fd):
@@ -119,7 +148,7 @@ def text_fd_to_metric_families(fd):
     so successful parsing does not imply that the parsed
     text meets the specification.
 
-    Yields core.Metric's.
+    Yields Metric's.
     """
     name = ''
     documentation = ''
@@ -128,7 +157,17 @@ def text_fd_to_metric_families(fd):
     allowed_names = []
 
     def build_metric(name, documentation, typ, samples):
-        metric = core.Metric(name, documentation, typ)
+        # Munge counters into OpenMetrics representation
+        # used internally.
+        if typ == 'counter':
+            if name.endswith('_total'):
+                name = name[:-6]
+            else:
+                new_samples = []
+                for s in samples:
+                    new_samples.append(Sample(s[0] + '_total', *s[1:]))
+                    samples = new_samples
+        metric = Metric(name, documentation, typ)
         metric.samples = samples
         return metric
 
@@ -176,7 +215,7 @@ def text_fd_to_metric_families(fd):
             pass
         else:
             sample = _parse_sample(line)
-            if sample[0] not in allowed_names:
+            if sample.name not in allowed_names:
                 if name != '':
                     yield build_metric(name, documentation, typ, samples)
                 # New metric, yield immediately as untyped singleton
